#![no_std]

extern crate alloc;

use alloc::boxed::Box;

/// # BlockDevice
/// interface for accessing a block device, i.e. a device that can be randomly accessed in fixed-size blocks
/// will be implemented in vapblock_* libraries
pub trait BlockDevice {
    /// moves the "cursor" to the given offset
    fn seek(&mut self, offset: usize);
    fn block_size(&self) -> usize;
    /// reads buf.len() bytes into buf, starting at the current offset
    /// returns the number of bytes read
    /// may return less than buf.len() if the end of the block device is reached,
    /// or if buf isn't a multiple of the block size
    fn read_blocks(&mut self, buf: &mut [u8]) -> usize;
    /// writes buf.len() bytes from buf, starting at the current offset
    /// returns the number of bytes written
    /// may return less than buf.len() if the end of the block device is reached
    fn write_blocks(&mut self, buf: &[u8]) -> usize;
}

/// # BasicROFile
/// interface for basic interactions with a file on a BasicROFileSystem
pub trait BasicROFile {
    /// moves the "cursor" to the given offset
    fn seek(&mut self, offset: usize);
    /// reads buf.len() bytes into buf, starting at the current offset
    /// returns the number of bytes read
    /// may return less than buf.len() if the end of the file is reached
    fn read(&mut self, buf: &mut [u8]) -> usize;
}

/// # BasicROFileSystem
/// interface for basic interactions with a filesystem over a block device
/// used in the booting process when there is likely only one task running and no need for
/// a more complex filesystem interface
pub trait BasicROFileSystem {
    /// must be called before any other methods
    fn init(&mut self, block_device: &mut dyn BlockDevice);
    /// checks if a given path exists
    fn exists(&self, path: &str) -> bool;
    /// checks if a given path is a directory
    fn is_dir(&self, path: &str) -> bool;
    /// checks if a given path is a file
    fn is_file(&self, path: &str) -> bool;
    /// checks if a given path is read-only and owned by root
    /// (returns true if the filesystem doesn't support permissions)
    fn is_safe(&self, path: &str) -> bool;
    /// opens a file at the given path
    /// returns None if the path doesn't exist or isn't a file
    fn open(&mut self, path: &str) -> Option<Box<dyn BasicROFile>>;
}